using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SensorTest.SensorDrivers;
using GHI.Pins;


namespace Test_Accel_Gyro
{
    public class Program
    {
        private static DateTime lastCheck = DateTime.MinValue;

        private static bool ITG3200(ITG3200 gyro)
        {
            if (gyro == null)
                return false;
            return gyro.Connected();
        }

        private static bool ADXL345(ADXL345 accel)
        {
            if (accel == null)
                return false;
            return accel.Connected();
        }

 

        private static string TimeStamp()
        {
            DateTime dt = DateTime.Now;
            if (lastCheck > DateTime.MinValue)
            {
                TimeSpan ts = dt.Subtract(lastCheck);
                return string.Concat(ts.Minutes, ":", ts.Seconds, ":", ts.Milliseconds);
            }
            else
            {
                return "0:0:0";
            }
        }

        public static void Main()
        {
            // Annoying the get the message when you want to see the values from the breakout boards
            // So set it to false..
            Debug.EnableGCMessages(true);
            bool ledState = false;

            OutputPort led = new OutputPort(FEZPandaIII.Gpio.Led1, ledState);

            ITG3200 gyro = new ITG3200(0x68, 400, 500, Cpu.Pin.GPIO_NONE);

            ADXL345 accel = new ADXL345(0x53, 400, 500, Cpu.Pin.GPIO_NONE);
         

            while (true)
            {

                lastCheck = DateTime.Now;
               
                if (ITG3200(gyro))
                {
                    Debug.Print("gyro");
                    Debug.Print("  " + TimeStamp() + ": x: " + gyro.X.ToString("f4"));
                    Debug.Print("  " + TimeStamp() + ": y: " + gyro.Y.ToString("f4"));
                    Debug.Print("  " + TimeStamp() + ": z: " + gyro.Z.ToString("f4"));
                    Debug.Print("  " + TimeStamp() + ": temperature: " + gyro.Temperature.ToString("f"));
                }

                if (ADXL345(accel))
                {
                    Debug.Print("Accel");
                    Debug.Print("  " + TimeStamp() + ": Xg: " + accel.ScaledXAxisG.ToString("F"));
                    Debug.Print("  " + TimeStamp() + ": Yg: " + accel.ScaledYAxisG.ToString("F"));
                    Debug.Print("  " + TimeStamp() + ": Zg: " + accel.ScaledZAxisG.ToString("F"));
                }

                Debug.Print("Done in " + TimeStamp());
                // Sleep for 500 milliseconds
                Thread.Sleep(500);

                // toggle LED state
                ledState = !ledState;
                led.Write(ledState);
            }
        }
    }
}
